@@ -1,5 +1,7 @@
 <?php
-/* Copyright (C) 2019-2023  Open-DSI    	    <support@open-dsi.fr>
+/* Copyright (C) 2019-2023  Open-DSI    	    		<support@open-dsi.fr>
+ * Copyright (C) 2024		Frédéric France				<frederic.france@free.fr>
+ * Copyright (C) 2025		Alexandre Spangaro			<alexandre@inovea-conseil.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,14 +31,22 @@ require_once DOL_DOCUMENT_ROOT.'/core/class/fiscalyear.class.php';
 require_once DOL_DOCUMENT_ROOT.'/core/class/html.formaccounting.class.php';
 require_once DOL_DOCUMENT_ROOT.'/accountancy/class/bookkeeping.class.php';
 
+/**
+ * @var Conf $conf
+ * @var DoliDB $db
+ * @var HookManager $hookmanager
+ * @var Translate $langs
+ * @var User $user
+ */
+
 // Load translation files required by the page
-$langs->loadLangs(array("compta", "bills", "other", "accountancy"));
+$langs->loadLangs(array("accountancy", "bills", "compta", "exports", "other"));
 
 $action = GETPOST('action', 'aZ09');
 $confirm = GETPOST('confirm', 'aZ09');
-$fiscal_period_id = GETPOST('fiscal_period_id', 'int');
-$validatemonth = GETPOST('validatemonth', 'int');
-$validateyear = GETPOST('validateyear', 'int');
+$fiscal_period_id = GETPOSTINT('fiscal_period_id');
+$validatemonth = GETPOSTINT('validatemonth');
+$validateyear = GETPOSTINT('validateyear');
 
 // Security check
 if (!isModEnabled('accounting')) {
@@ -49,7 +59,7 @@ if (!$user->hasRight('accounting', 'fiscalyear', 'write')) {
 	accessforbidden();
 }
 
-// Initialize technical object to manage hooks of page. Note that conf->hooks_modules contains array of hook context
+// Initialize a technical object to manage hooks of page. Note that conf->hooks_modules contains an array of hook context
 $hookmanager->initHooks(array('accountancyclosure'));
 
 $object = new BookKeeping($db);
@@ -126,10 +136,10 @@ if ($reshook < 0) {
 }
 
 if (empty($reshook)) {
-	if (isset($current_fiscal_period) && $user->hasRight('accounting', 'fiscalyear', 'write')) {
-		if ($action == 'confirm_step_1' && $confirm == "yes") {
-			$date_start = dol_mktime(0, 0, 0, GETPOST('date_startmonth', 'int'), GETPOST('date_startday', 'int'), GETPOST('date_startyear', 'int'));
-			$date_end = dol_mktime(23, 59, 59, GETPOST('date_endmonth', 'int'), GETPOST('date_endday', 'int'), GETPOST('date_endyear', 'int'));
+	if (isset($current_fiscal_period)) {
+		if ($action == 'confirm_step_1' && $confirm == "yes" && $user->hasRight('accounting', 'fiscalyear', 'write')) {
+			$date_start = dol_mktime(0, 0, 0, GETPOSTINT('date_startmonth'), GETPOSTINT('date_startday'), GETPOSTINT('date_startyear'));
+			$date_end = dol_mktime(23, 59, 59, GETPOSTINT('date_endmonth'), GETPOSTINT('date_endday'), GETPOSTINT('date_endyear'));
 
 			$result = $object->validateMovementForFiscalPeriod($date_start, $date_end);
 			if ($result > 0) {
@@ -142,8 +152,8 @@ if (empty($reshook)) {
 				setEventMessages($object->error, $object->errors, 'errors');
 				$action = '';
 			}
-		} elseif ($action == 'confirm_step_2' && $confirm == "yes") {
-			$new_fiscal_period_id = GETPOST('new_fiscal_period_id', 'int');
+		} elseif ($action == 'confirm_step_2' && $confirm == "yes" && $user->hasRight('accounting', 'fiscalyear', 'write')) {
+			$new_fiscal_period_id = GETPOSTINT('new_fiscal_period_id');
 			$separate_auxiliary_account = GETPOST('separate_auxiliary_account', 'aZ09');
 			$generate_bookkeeping_records = GETPOST('generate_bookkeeping_records', 'aZ09');
 
@@ -169,11 +179,11 @@ if (empty($reshook)) {
 					exit;
 				}
 			}
-		} elseif ($action == 'confirm_step_3' && $confirm == "yes") {
-			$inventory_journal_id = GETPOST('inventory_journal_id', 'int');
-			$new_fiscal_period_id = GETPOST('new_fiscal_period_id', 'int');
-			$date_start = dol_mktime(0, 0, 0, GETPOST('date_startmonth', 'int'), GETPOST('date_startday', 'int'), GETPOST('date_startyear', 'int'));
-			$date_end = dol_mktime(23, 59, 59, GETPOST('date_endmonth', 'int'), GETPOST('date_endday', 'int'), GETPOST('date_endyear', 'int'));
+		} elseif ($action == 'confirm_step_3' && $confirm == "yes" && $user->hasRight('accounting', 'fiscalyear', 'write')) {
+			$inventory_journal_id = GETPOSTINT('inventory_journal_id');
+			$new_fiscal_period_id = GETPOSTINT('new_fiscal_period_id');
+			$date_start = dol_mktime(0, 0, 0, GETPOSTINT('date_startmonth'), GETPOSTINT('date_startday'), GETPOSTINT('date_startyear'));
+			$date_end = dol_mktime(23, 59, 59, GETPOSTINT('date_endmonth'), GETPOSTINT('date_endday'), GETPOSTINT('date_endyear'));
 
 			$result = $object->insertAccountingReversal($current_fiscal_period['id'], $inventory_journal_id, $new_fiscal_period_id, $date_start, $date_end);
 			if ($result < 0) {
@@ -198,9 +208,9 @@ $formaccounting = new FormAccounting($db);
 
 $title = $langs->trans('Closure');
 
-$help_url ='EN:Module_Double_Entry_Accounting|FR:Module_Comptabilit&eacute;_en_Partie_Double#Cl.C3.B4ture_annuelle';
+$help_url = 'EN:Module_Double_Entry_Accounting|FR:Module_Comptabilit&eacute;_en_Partie_Double#Cl.C3.B4ture_annuelle';
 
-llxHeader('', $title, $help_url);
+llxHeader('', $title, $help_url, '', 0, 0, '', '', '', 'mod-accountancy page-closure-index');
 
 $formconfirm = '';
 
@@ -229,7 +239,8 @@ if (isset($current_fiscal_period)) {
 			$form_question,
 			'',
 			1,
-			300
+			300,
+			600
 		);
 	} elseif ($action == 'step_2') {
 		$form_question = array();
@@ -266,7 +277,8 @@ if (isset($current_fiscal_period)) {
 			$form_question,
 			'',
 			1,
-			300
+			300,
+			600
 		);
 	} elseif ($action == 'step_3') {
 		$form_question = array();
@@ -292,7 +304,7 @@ if (isset($current_fiscal_period)) {
 			'name' => 'date_start',
 			'type' => 'date',
 			'label' => $langs->trans('DateStart'),
-			'value' => dol_time_plus_duree($current_fiscal_period['date_end'], -1, 'm')
+			'value' => dol_time_plus_duree((int) $current_fiscal_period['date_end'], -1, 'm')
 		);
 		$form_question['date_end'] = array(
 			'name' => 'date_end',
@@ -309,7 +321,8 @@ if (isset($current_fiscal_period)) {
 			$form_question,
 			'',
 			1,
-			300
+			300,
+			600
 		);
 	}
 }
@@ -337,90 +350,102 @@ if (!empty($current_fiscal_period)) {
 print load_fiche_titre($langs->trans("Closure") . " - " . $fiscal_period_nav_text, '', 'title_accountancy');
 
 if (empty($current_fiscal_period)) {
-	print $langs->trans('ErrorNoFiscalPeriodActiveFound');
+	print $langs->trans('ErrorNoFiscalPeriodActiveFound', $langs->transnoentitiesnoconv("Accounting"), $langs->transnoentitiesnoconv("Setup"), $langs->transnoentitiesnoconv("FiscalPeriod"));
 } else {
-	// Step 1
-	$head = array();
-	$head[0][0] = DOL_URL_ROOT . '/accountancy/closure/index.php?fiscal_period_id=' . $current_fiscal_period['id'];
-	$head[0][1] = $langs->trans("AccountancyClosureStep1");
-	$head[0][2] = 'step1';
-	print dol_get_fiche_head($head, 'step1', '', -1, 'title_accountancy');
+	if (!getDolGlobalString("ACCOUNTANCY_DISABLE_CLOSURE_LINE_BY_LINE")) {
+		// Step 1
+		$head = array();
+		$head[0][0] = DOL_URL_ROOT . '/accountancy/closure/index.php?fiscal_period_id=' . $current_fiscal_period['id'];
+		$head[0][1] = $langs->trans("Step").' 1 - '.$langs->trans("AccountancyClosureStep1");
+		$head[0][2] = 'step1';
+		print dol_get_fiche_head($head, 'step1', '', -1, '');
 
-	print '<span class="opacitymedium">' . $langs->trans("AccountancyClosureStep1Desc") . '</span><br>';
+		print '<span class="opacitymedium">' . $langs->trans("AccountancyClosureStep1Desc") . '</span><br>';
 
-	$count_by_month = $object->getCountByMonthForFiscalPeriod($current_fiscal_period['date_start'], $current_fiscal_period['date_end']);
-	if (!is_array($count_by_month)) {
-		setEventMessages($object->error, $object->errors, 'errors');
-	}
+		$count_by_month = $object->getCountByMonthForFiscalPeriod($current_fiscal_period['date_start'], $current_fiscal_period['date_end']);
 
-	if (empty($count_by_month['total'])) {
-		$buttonvalidate = '<a class="butActionRefused classfortooltip" href="#">' . $langs->trans("ValidateMovements") . '</a>';
-	} else {
-		$buttonvalidate = '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?action=step_1&fiscal_period_id=' . $current_fiscal_period['id'] . '">' . $langs->trans("ValidateMovements") . '</a>';
-	}
-	print_barre_liste($langs->trans("OverviewOfMovementsNotValidated"), '', '', '', '', '', '', -1, '', '', 0, $buttonvalidate, '', 0, 1, 0);
-
-	print '<div class="div-table-responsive-no-min">';
-	print '<table class="noborder centpercent">';
-
-	print '<tr class="liste_titre">';
-	$nb_years = is_array($count_by_month['list']) ? count($count_by_month['list']) : 0;
-	if ($nb_years > 1) {
-		print '<td class="right">' . $langs->trans("Year") . '</td>';
-	}
-	for ($i = 1; $i <= 12; $i++) {
-		print '<td class="right">' . $langs->trans('MonthShort' . str_pad($i, 2, '0', STR_PAD_LEFT)) . '</td>';
-	}
-	print '<td class="right"><b>' . $langs->trans("Total") . '</b></td>';
-	print '</tr>';
-
-	if (is_array($count_by_month['list'])) {
-		foreach ($count_by_month['list'] as $info) {
-			print '<tr class="oddeven">';
-			if ($nb_years > 1) {
-				print '<td class="right">' . $info['year'] . '</td>';
-			}
-			for ($i = 1; $i <= 12; $i++) {
-				print '<td class="right">' . ((int) $info['count'][$i]) . '</td>';
-			}
-			print '<td class="right"><b>' . $info['total'] . '</b></td></tr>';
+		if (!is_array($count_by_month)) {
+			setEventMessages($object->error, $object->errors, 'errors');
 		}
-	}
 
-	print "</table>\n";
-	print '</div>';
+		if (empty($count_by_month['total'])) {
+			$buttonvalidate = '<a class="butActionRefused classfortooltip" href="#">' . $langs->trans("ValidateMovements") . '</a>';
+		} else {
+			$buttonvalidate = '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?action=step_1&token='.newToken().'&fiscal_period_id=' . $current_fiscal_period['id'] . '">' . $langs->trans("ValidateMovements") . '</a>';
+		}
+		print_barre_liste($langs->trans("OverviewOfMovementsNotValidated"), 0, '', '', '', '', '', -1, '', '', 0, $buttonvalidate, '', 0, 1, 0);
+
+		print '<div class="div-table-responsive-no-min">';
+		print '<table class="noborder centpercent">';
+
+		print '<tr class="liste_titre">';
+		$nb_years = is_array($count_by_month['list']) ? count($count_by_month['list']) : 0;
+		if ($nb_years > 1) {
+			print '<td class="right">' . $langs->trans("Year") . '</td>';
+		}
+		for ($i = 1; $i <= 12; $i++) {
+			print '<td class="right">' . $langs->trans('MonthShort' . str_pad((string) $i, 2, '0', STR_PAD_LEFT)) . '</td>';
+		}
+		print '<td class="right"><b>' . $langs->trans("Total") . '</b></td>';
+		print '</tr>';
+
+		if (is_array($count_by_month['list'])) {
+			foreach ($count_by_month['list'] as $info) {
+				print '<tr class="oddeven">';
+				if ($nb_years > 1) {
+					print '<td class="right">' . $info['year'] . '</td>';
+				}
+				for ($i = 1; $i <= 12; $i++) {
+					print '<td class="right">' . ((int) $info['count'][$i]) . '</td>';
+				}
+				print '<td class="right"><b>' . $info['total'] . '</b></td></tr>';
+			}
+		}
+
+		print "</table>\n";
+		print '</div>';
+
+		print '<br>';
+	}
 
 	// Step 2
 	$head = array();
 	$head[0][0] = DOL_URL_ROOT . '/accountancy/closure/index.php?fiscal_period_id=' . $current_fiscal_period['id'];
-	$head[0][1] = $langs->trans("AccountancyClosureStep2");
+	$head[0][1] = $langs->trans("Step"). ' ' . (getDolGlobalString("ACCOUNTANCY_DISABLE_CLOSURE_LINE_BY_LINE") ? '1' : '2').' - '.$langs->trans("AccountancyClosureStep2");
 	$head[0][2] = 'step2';
-	print dol_get_fiche_head($head, 'step2', '', -1, 'title_accountancy');
+	print dol_get_fiche_head($head, 'step2', '', -1, '');
 
+	$button = '';
 	// print '<span class="opacitymedium">' . $langs->trans("AccountancyClosureStep2Desc") . '</span><br>';
-
-	if (empty($count_by_month['total']) && empty($current_fiscal_period['status'])) {
-		$button = '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?action=step_2&fiscal_period_id=' . $current_fiscal_period['id'] . '">' . $langs->trans("AccountancyClosureClose") . '</a>';
+	if ((empty($count_by_month['total']) || getDolGlobalString("ACCOUNTANCY_DISABLE_CLOSURE_LINE_BY_LINE")) && empty($current_fiscal_period['status'])) {
+		// If no unlocked record and period still open
+		$button = '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?action=step_2&token='.newToken().'&fiscal_period_id=' . $current_fiscal_period['id'] . '">' . $langs->trans("AccountancyClosureClose") . '</a>';
 	} else {
-		$button = '<a class="butActionRefused classfortooltip" href="#" title="This fiscal period already has the status Closed. Feature disabled.">' . $langs->trans("AccountancyClosureClose") . '</a>';
+		if (!empty($current_fiscal_period['status'])) {
+			$button = '<a class="butActionRefused classfortooltip" href="#" title="The period is already closed. Feature disabled.">' . $langs->trans("AccountancyClosureClose") . '</a>';
+		} elseif (!empty($count_by_month['total'])) {
+			$button = '<a class="butActionRefused classfortooltip" href="#" title="There is some lines not yet locked. Feature disabled.">' . $langs->trans("AccountancyClosureClose") . '</a>';
+		}
 	}
-	print_barre_liste('', '', '', '', '', '', '', -1, '', '', 0, $button, '', 0, 1, 0);
+	print_barre_liste('', 0, '', '', '', '', '', -1, '', '', 0, $button, '', 0, 1, 0);
+
+	print '<br>';
 
 	// Step 3
 	$head = array();
 	$head[0][0] = DOL_URL_ROOT . '/accountancy/closure/index.php?fiscal_period_id=' . $current_fiscal_period['id'];
-	$head[0][1] = $langs->trans("AccountancyClosureStep3");
+	$head[0][1] = $langs->trans("Step"). ' ' . (getDolGlobalString("ACCOUNTANCY_DISABLE_CLOSURE_LINE_BY_LINE") ? '2' : '3').' - '.$langs->trans("AccountancyClosureStep3");
 	$head[0][2] = 'step3';
-	print dol_get_fiche_head($head, 'step3', '', -1, 'title_accountancy');
+	print dol_get_fiche_head($head, 'step3', '', -1, '');
 
 	// print '<span class="opacitymedium">' . $langs->trans("AccountancyClosureStep3Desc") . '</span><br>';
 
 	if (empty($current_fiscal_period['status'])) {
 		$button = '<a class="butActionRefused classfortooltip" href="#">' . $langs->trans("AccountancyClosureAccountingReversal") . '</a>';
 	} else {
-		$button = '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?action=step_3&fiscal_period_id=' . $current_fiscal_period['id'] . '">' . $langs->trans("AccountancyClosureAccountingReversal") . '</a>';
+		$button = '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?action=step_3&token='.newToken().'&fiscal_period_id=' . $current_fiscal_period['id'] . '">' . $langs->trans("AccountancyClosureAccountingReversal") . '</a>';
 	}
-	print_barre_liste('', '', '', '', '', '', '', -1, '', '', 0, $button, '', 0, 1, 0);
+	print_barre_liste('', 0, '', '', '', '', '', -1, '', '', 0, $button, '', 0, 1, 0);
 }
 
 // End of page
