@@ -4,7 +4,7 @@
  * Copyright (C) 2011-2015 Juanjo Menent        <jmenent@2byte.es>
  * Copyright (C) 2017      Ferran Marcet        <fmarcet@2byte.es>
  * Copyright (C) 2018-2024 Charlene Benke       <charlene@patas-monkey.com>
- * Copyright (C) 2024	     MDW					        <mdeweerd@users.noreply.github.com>
+ * Copyright (C) 2024		MDW							<mdeweerd@users.noreply.github.com>
  * Copyright (C) 2024      Frédéric France      <frederic.france@free.fr>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,7 @@
 /**
  *  Return an array with timezone values
  *
- *  @return     array   Array with timezone values
+ *  @return     array<int<-11,13>,string>   Array with timezone values
  */
 function get_tz_array()
 {
@@ -105,7 +105,7 @@ function getServerTimeZoneInt($refgmtdate = 'now')
 		//print $refgmtdate.'='.$tmp;
 	} else {
 		$tmp = 0;
-		dol_print_error('', 'PHP version must be 5.3+');
+		dol_print_error(null, 'PHP version must be 5.3+');
 	}
 	$tz = round(($tmp < 0 ? 1 : -1) * abs($tmp / 3600));
 	return $tz;
@@ -115,34 +115,35 @@ function getServerTimeZoneInt($refgmtdate = 'now')
 /**
  *  Add a delay to a date
  *
- *  @param      int			$time               Date timestamp (or string with format YYYY-MM-DD)
- *  @param      int			$duration_value     Value of delay to add
- *  @param      int			$duration_unit      Unit of added delay (d, m, y, w, h, i)
- *  @param      int         $ruleforendofmonth  Change the behavior of PHP over data-interval, 0 or 1
+ *  @param      int|string	$time               Date timestamp (or string with format YYYY-MM-DD)
+ *  @param      float		$duration_value     Value of delay to add
+ *  @param      string		$duration_unit      Unit of added delay (d, m, y, w, h, i)
+ *  @param      int<0,1>    $ruleforendofmonth  Change the behavior of PHP over data-interval, 0 or 1
  *  @return     int      			        	New timestamp
  *  @see convertSecondToTime(), convertTimeToSeconds()
  */
 function dol_time_plus_duree($time, $duration_value, $duration_unit, $ruleforendofmonth = 0)
 {
-	global $conf;
 	if (empty($duration_value)) {
 		return $time;
 	}
 	if ($duration_unit == 's') {
-		return $time + ($duration_value);
+		return $time + (int) ($duration_value);
 	}
-	if ($duration_unit == 'i') {
-		return $time + (60 * $duration_value);
+	if ($duration_unit == 'i' || $duration_unit == 'mn' || $duration_unit == 'min') {
+		return $time + (int) (60 * $duration_value);
 	}
 	if ($duration_unit == 'h') {
-		return $time + (3600 * $duration_value);
+		return $time + (int) (3600 * $duration_value);
 	}
 	if ($duration_unit == 'w') {
-		return $time + (3600 * 24 * 7 * $duration_value);
+		return $time + (int) (3600 * 24 * 7 * $duration_value);
 	}
 
 	$deltastring = 'P';
 
+	$sub = false;
+
 	if ($duration_value > 0) {
 		$deltastring .= abs($duration_value);
 		$sub = false;
@@ -165,7 +166,7 @@ function dol_time_plus_duree($time, $duration_value, $duration_unit, $ruleforend
 	if (getDolGlobalString('MAIN_DATE_IN_MEMORY_ARE_GMT')) {
 		$date->setTimezone(new DateTimeZone('UTC'));
 	}
-	$date->setTimestamp($time);
+	$date->setTimestamp((int) $time);
 	$interval = new DateInterval($deltastring);
 
 	if ($sub) {
@@ -175,22 +176,22 @@ function dol_time_plus_duree($time, $duration_value, $duration_unit, $ruleforend
 	}
 	//Change the behavior of PHP over data-interval when the result of this function is Feb 29 (non-leap years), 30 or Feb 31 (so php returns March 1, 2 or 3 respectively)
 	if ($ruleforendofmonth == 1 && $duration_unit == 'm') {
-		$timeyear = dol_print_date($time, '%Y');
-		$timemonth = dol_print_date($time, '%m');
+		$timeyear = (int) dol_print_date($time, '%Y');
+		$timemonth = (int) dol_print_date($time, '%m');
 		$timetotalmonths = (($timeyear * 12) + $timemonth);
 
 		$monthsexpected = ($timetotalmonths + $duration_value);
 
 		$newtime = $date->getTimestamp();
 
-		$newtimeyear = dol_print_date($newtime, '%Y');
-		$newtimemonth = dol_print_date($newtime, '%m');
-		$newtimetotalmonths = (($newtimeyear * 12) + $newtimemonth);
+		$newtimeyear = (int) dol_print_date($newtime, '%Y');
+		$newtimemonth = (int) dol_print_date($newtime, '%m');
+		$newtimetotalmonths = (($newtimeyear * 12) +  $newtimemonth);
 
 		if ($monthsexpected < $newtimetotalmonths) {
-			$newtimehours = dol_print_date($newtime, '%H');
-			$newtimemins = dol_print_date($newtime, '%M');
-			$newtimesecs = dol_print_date($newtime, '%S');
+			$newtimehours = (int) dol_print_date($newtime, '%H');
+			$newtimemins = (int) dol_print_date($newtime, '%M');
+			$newtimesecs = (int) dol_print_date($newtime, '%S');
 
 			$datelim = dol_mktime($newtimehours, $newtimemins, $newtimesecs, $newtimemonth, 1, $newtimeyear);
 			$datelim -= (3600 * 24);
@@ -222,21 +223,21 @@ function convertTime2Seconds($iHours = 0, $iMinutes = 0, $iSeconds = 0)
  *      Can be used to show a duration.
  *
  *    	@param      int		$iSecond		Number of seconds
- *    	@param      string	$format		    Output format
- *    										- 'all': total delay days hour:min like "2 days 12:30",
- *                                          - 'allwithouthour': total delay days without hour part like "2 days",
- *                                          - 'allhourmin': total delay with format hours:min like "60:30",
- *                                          - 'allhourminsec': total delay with format hours:min:sec like "60:30:10",
- *                                          - 'allhour': total delay hours without min/sec like "60:30",
- *                                          - 'fullhour': total delay hour decimal like "60.5" for 60:30,
- *                                          - 'hour': only hours part "12",
- *                                          - 'min': only minutes part "30",
- *                                          - 'sec': only seconds part,
- *                                          - 'month': only month part,
- *                                          - 'year': only year part);
- *      @param      int		$lengthOfDay    Length of day (default 86400 seconds for 1 day, 28800 for 8 hour)
- *      @param      int		$lengthOfWeek   Length of week (default 7)
- *    	@return     string		 		 	Formated text of duration
+ *		@param		'all'|'allwithouthour'|'allhourmin'|'allhourminsec'|'allhour'|'fullhour'|'hour'|'min'|'sec'|'month'|'year'	$format		    Output format
+ *                                                                                                                                              - 'all': total delay days hour:min like "2 days 12:30",
+ *                                                                                                                                              - 'allwithouthour': total delay days without hour part like "2 days",
+ *                                                                                                                                              - 'allhourmin': total delay with format hours:min like "60:30",
+ *                                                                                                                                              - 'allhourminsec': total delay with format hours:min:sec like "60:30:10",
+ *                                                                                                                                              - 'allhour': total delay hours without min/sec like "60:30",
+ *                                                                                                                                              - 'fullhour': total delay hour decimal like "60.5" for 60:30,
+ *                                                                                                                                              - 'hour': only hours part "12",
+ *                                                                                                                                              - 'min': only minutes part "30",
+ *                                                                                                                                              - 'sec': only seconds part,
+ *                                                                                                                                              - 'month': only month part,
+ *                                                                                                                                              - 'year': only year part);
+ *      @param      int<1,86400>	$lengthOfDay    Length of day (default 86400 seconds for 1 day, 28800 for 8 hour)
+ *      @param      int<1,7>	$lengthOfWeek   Length of week (default 7)
+ *    	@return     string		 		 	Formatted text of duration
  * 	                                		Example: 0 return 00:00, 3600 return 1:00, 86400 return 1d, 90000 return 1 Day 01:00
  *      @see convertTime2Seconds()
  */
@@ -252,12 +253,13 @@ function convertSecondToTime($iSecond, $format = 'all', $lengthOfDay = 86400, $l
 	}
 	$nbHbyDay = $lengthOfDay / 3600;
 
+	$sTime = '';
+
 	if ($format == 'all' || $format == 'allwithouthour' || $format == 'allhour' || $format == 'allhourmin' || $format == 'allhourminsec') {
 		if ((int) $iSecond === 0) {
 			return '0'; // This is to avoid having 0 return a 12:00 AM for en_US
 		}
 
-		$sTime = '';
 		$sDay = 0;
 		$sWeek = 0;
 
@@ -276,7 +278,7 @@ function convertSecondToTime($iSecond, $format = 'all', $lengthOfDay = 86400, $l
 			if ($sDay) {
 				if ($sDay >= $lengthOfWeek) {
 					$sWeek = (int) (($sDay - $sDay % $lengthOfWeek) / $lengthOfWeek);
-					$sDay = $sDay % $lengthOfWeek;
+					$sDay %= $lengthOfWeek;
 					$weekTranslate = $langs->trans("DurationWeek");
 					if ($sWeek >= 2) {
 						$weekTranslate = $langs->trans("DurationWeeks");
@@ -298,17 +300,17 @@ function convertSecondToTime($iSecond, $format = 'all', $lengthOfDay = 86400, $l
 				$sTime .= dol_print_date($iSecond, 'hourduration', true);
 			}
 		} elseif ($format == 'allhourminsec') {
-			return sprintf("%02d", ($sWeek * $lengthOfWeek * $nbHbyDay + $sDay * $nbHbyDay + (int) floor($iSecond/3600))).':'.sprintf("%02d", ((int) floor(($iSecond % 3600) / 60))).':'.sprintf("%02d", ((int) ($iSecond % 60)));
+			return sprintf("%02d", ($sWeek * $lengthOfWeek * $nbHbyDay + $sDay * $nbHbyDay + (int) floor($iSecond / 3600))).':'.sprintf("%02d", ((int) floor(($iSecond % 3600) / 60))).':'.sprintf("%02d", ((int) ($iSecond % 60)));
 		} elseif ($format == 'allhourmin') {
-			return sprintf("%02d", ($sWeek * $lengthOfWeek * $nbHbyDay + $sDay * $nbHbyDay + (int) floor($iSecond/3600))).':'.sprintf("%02d", ((int) floor(($iSecond % 3600)/60)));
+			return sprintf("%02d", ($sWeek * $lengthOfWeek * $nbHbyDay + $sDay * $nbHbyDay + (int) floor($iSecond / 3600))).':'.sprintf("%02d", ((int) floor(($iSecond % 3600) / 60)));
 		} elseif ($format == 'allhour') {
-			return sprintf("%02d", ($sWeek * $lengthOfWeek * $nbHbyDay + $sDay * $nbHbyDay + (int) floor($iSecond/3600)));
+			return sprintf("%02d", ($sWeek * $lengthOfWeek * $nbHbyDay + $sDay * $nbHbyDay + (int) floor($iSecond / 3600)));
 		}
 	} elseif ($format == 'hour') {	// only hour part
 		$sTime = dol_print_date($iSecond, '%H', true);
 	} elseif ($format == 'fullhour') {
 		if (!empty($iSecond)) {
-			$iSecond = $iSecond / 3600;
+			$iSecond /= 3600;
 		} else {
 			$iSecond = 0;
 		}
@@ -322,15 +324,16 @@ function convertSecondToTime($iSecond, $format = 'all', $lengthOfDay = 86400, $l
 	} elseif ($format == 'year') {	// only year part
 		$sTime = dol_print_date($iSecond, '%Y', true);
 	}
-	return trim($sTime);
+	return trim((string) $sTime);
 }
 
 
 /**	  	Convert duration to hour
  *
- *    	@param      int		$duration_value		Duration value
- *    	@param      int		$duration_unit		Duration unit
- *      @return     int $result
+ *    	@param      float	$duration_value		Duration value
+ *    	@param      string	$duration_unit		Duration unit
+ *      @return     float	$result
+ *      @see measuringUnitString()
  */
 function convertDurationtoHour($duration_value, $duration_unit)
 {
@@ -339,7 +342,7 @@ function convertDurationtoHour($duration_value, $duration_unit)
 	if ($duration_unit == 's') {
 		$result = $duration_value / 3600;
 	}
-	if ($duration_unit == 'i') {
+	if ($duration_unit == 'i' || $duration_unit == 'mn' || $duration_unit == 'min') {
 		$result = $duration_value / 60;
 	}
 	if ($duration_unit == 'h') {
@@ -371,8 +374,10 @@ function convertDurationtoHour($duration_value, $duration_unit)
  * @param      int|string	$year_date			Year date
  * @param	   int      	$excludefirstand	Exclude first and
  * @param	   mixed		$gm					False or 0 or 'tzserver' = Input date fields are date info in the server TZ. True or 1 or 'gmt' = Input are date info in GMT TZ.
- * 												Note: In database, dates are always fot the server TZ.
+ * 												Note: In database, dates are always for the server TZ.
  * @return     string		$sqldate			String with SQL filter
+ * @see forgeSQLFromUniversalSearchCriteria()
+ * @see natural_search()
  */
 function dolSqlDateFilter($datefield, $day_date, $month_date, $year_date, $excludefirstand = 0, $gm = false)
 {
@@ -431,12 +436,12 @@ function dol_stringtotime($string, $gm = 1)
 		dol_syslog("dol_stringtotime call to function with deprecated parameter format", LOG_WARNING);
 		// Date est au format 'DD/MM/YY' ou 'DD/MM/YY HH:MM:SS'
 		// Date est au format 'DD/MM/YYYY' ou 'DD/MM/YYYY HH:MM:SS'
-		$sday = $reg[1];
-		$smonth = $reg[2];
-		$syear = $reg[3];
-		$shour = $reg[4];
-		$smin = $reg[5];
-		$ssec = $reg[6];
+		$sday = (int) $reg[1];
+		$smonth = (int) $reg[2];
+		$syear = (int) $reg[3];
+		$shour = (int) $reg[4];
+		$smin = (int) $reg[5];
+		$ssec = (int) $reg[6];
 		if ($syear < 50) {
 			$syear += 1900;
 		}
@@ -448,12 +453,12 @@ function dol_stringtotime($string, $gm = 1)
 		|| preg_match('/^([0-9]{4})-([0-9]{2})-([0-9]{2}) ([0-9]{2}):([0-9]{2}):([0-9]{2})$/i', $string, $reg)	// Convert date with format YYYY-MM-DD HH:MM:SS
 		|| preg_match('/^([0-9]{4})([0-9]{2})([0-9]{2})T([0-9]{2})([0-9]{2})([0-9]{2})Z$/i', $string, $reg)		// Convert date with format YYYYMMDDTHHMMSSZ
 	) {
-		$syear = $reg[1];
-		$smonth = $reg[2];
-		$sday = $reg[3];
-		$shour = $reg[4];
-		$smin = $reg[5];
-		$ssec = $reg[6];
+		$syear = (int) $reg[1];
+		$smonth = (int) $reg[2];
+		$sday = (int) $reg[3];
+		$shour = (int) $reg[4];
+		$smin = (int) $reg[5];
+		$ssec = (int) $reg[6];
 		$string = sprintf("%04d%02d%02d%02d%02d%02d", $syear, $smonth, $sday, $shour, $smin, $ssec);
 	}
 
@@ -466,7 +471,8 @@ function dol_stringtotime($string, $gm = 1)
 		$gm = 'tzserver';
 	}
 
-	$date = dol_mktime(substr($tmp, 8, 2), substr($tmp, 10, 2), substr($tmp, 12, 2), substr($tmp, 4, 2), substr($tmp, 6, 2), substr($tmp, 0, 4), $gm);
+	$date = dol_mktime((int) substr($tmp, 8, 2), (int) substr($tmp, 10, 2), (int) substr($tmp, 12, 2), (int) substr($tmp, 4, 2), (int) substr($tmp, 6, 2), (int) substr($tmp, 0, 4), $gm);
+
 	return $date;
 }
 
@@ -477,7 +483,7 @@ function dol_stringtotime($string, $gm = 1)
  *  @param      int			$day     	Day
  *  @param      int			$month   	Month
  *  @param      int			$year    	Year
- *  @return     array   				Previous year,month,day
+ *  @return     array{year:int,month:int,day:int}	Previous year,month,day
  */
 function dol_get_prev_day($day, $month, $year)
 {
@@ -493,7 +499,7 @@ function dol_get_prev_day($day, $month, $year)
  *  @param      int			$day    	Day
  *  @param      int			$month  	Month
  *  @param      int			$year   	Year
- *  @return     array   				Next year,month,day
+ *  @return     array{year:int,month:int,day:int}	Next year,month,day
  */
 function dol_get_next_day($day, $month, $year)
 {
@@ -508,7 +514,7 @@ function dol_get_next_day($day, $month, $year)
  *
  *	@param		int			$month		Month
  *	@param		int			$year		Year
- *	@return		array					Previous year,month
+ *	@return		array{year:int,month:int}	Previous year,month
  */
 function dol_get_prev_month($month, $year)
 {
@@ -527,7 +533,7 @@ function dol_get_prev_month($month, $year)
  *
  *	@param		int			$month		Month
  *	@param		int			$year		Year
- *	@return		array					Next year,month
+ *	@return		array{year:int,month:int}	Next year,month
  */
 function dol_get_next_month($month, $year)
 {
@@ -548,7 +554,7 @@ function dol_get_next_month($month, $year)
  *  @param      int			$week    	Week
  *  @param      int			$month   	Month
  *	@param		int			$year		Year
- *	@return		array					Previous year,month,day
+ *	@return		array{year:int,month:int,day:int}	Previous year,month,day
  */
 function dol_get_prev_week($day, $week, $month, $year)
 {
@@ -567,7 +573,7 @@ function dol_get_prev_week($day, $week, $month, $year)
  *  @param      int			$week    	Week
  *  @param      int			$month   	Month
  *	@param		int			$year		Year
- *	@return		array					Next year,month,day
+ *	@return		array{year:int,month:int,day:int}		Next year,month,day
  */
 function dol_get_next_week($day, $week, $month, $year)
 {
@@ -585,11 +591,11 @@ function dol_get_next_week($day, $week, $month, $year)
  *
  *	@param		int			$year		Year
  * 	@param		int			$month		Month
- * 	@param		mixed		$gm			False or 0 or 'tzserver' = Return date to compare with server TZ,
+ * 	@param		bool|int|string	$gm		False or 0 or 'tzserver' = Return date to compare with server TZ,
  * 										True or 1 or 'gmt' to compare with GMT date.
  *                          			Example: dol_get_first_day(1970,1,false) will return -3600 with TZ+1, a dol_print_date on it will return 1970-01-01 00:00:00
  *                          			Example: dol_get_first_day(1970,1,true) will return 0 whatever is TZ, a dol_print_date on it will return 1970-01-01 00:00:00
- *  @return		int						Date for first day, '' if error
+ *  @return		int|string				Date as a timestamp, '' if error
  */
 function dol_get_first_day($year, $month = 1, $gm = false)
 {
@@ -606,9 +612,9 @@ function dol_get_first_day($year, $month = 1, $gm = false)
  *
  *	@param		int			$year		Year
  * 	@param		int			$month		Month
- * 	@param		mixed		$gm			False or 0 or 'tzserver' = Return date to compare with server TZ,
+ * 	@param		bool|int|string	$gm		False or 0 or 'tzserver' = Return date to compare with server TZ,
  * 										True or 1 or 'gmt' to compare with GMT date.
- *	@return		int						Date for first day, '' if error
+ *	@return		int|string				Date as a timestamp, '' if error
  */
 function dol_get_last_day($year, $month = 12, $gm = false)
 {
@@ -633,7 +639,7 @@ function dol_get_last_day($year, $month = 12, $gm = false)
  *  Return GMT time for last hour of a given GMT date (it replaces hours, min and second part to 23:59:59)
  *
  *	@param		int			$date		Date GMT
- * 	@param		mixed		$gm			False or 0 or 'tzserver' = Return date to compare with server TZ,
+ * 	@param		bool|int|string	$gm		False or 0 or 'tzserver' = Return date to compare with server TZ,
  * 										'gmt' to compare with GMT date.
  *  @return		int						Date for last hour of a given date
  */
@@ -647,9 +653,9 @@ function dol_get_last_hour($date, $gm = 'tzserver')
  *  Return GMT time for first hour of a given GMT date (it removes hours, min and second part)
  *
  *	@param		int			$date		Date GMT
- * 	@param		mixed		$gm			False or 0 or 'tzserver' = Return date to compare with server TZ,
+ * 	@param		bool|int|string	$gm		False or 0 or 'tzserver' = Return date to compare with server TZ,
  * 										'gmt' to compare with GMT date.
- *  @return		int						Date for last hour of a given date
+ *  @return		int						Date for first hour of a given date
  */
 function dol_get_first_hour($date, $gm = 'tzserver')
 {
@@ -662,9 +668,9 @@ function dol_get_first_hour($date, $gm = 'tzserver')
  *	@param		int		$day		Day
  * 	@param		int		$month		Month
  *  @param		int		$year		Year
- * 	@param		mixed	$gm			False or 0 or 'tzserver' = Return date to compare with server TZ,
- * 									True or 1 or 'gmt' to compare with GMT date.
- *	@return		array				year,month,week,first_day,first_month,first_year,prev_day,prev_month,prev_year
+ * 	@param		bool|int|'tzserver'	$gm	False or 0 or 'tzserver' = Return date to compare with server TZ,
+ *                                      True or 1 or 'gmt' to compare with GMT date.
+ *	@return	array{year:int,month:int,week:string,first_day:int,first_month:int,first_year:int,prev_year:int,prev_month:int,prev_day:int}
  */
 function dol_get_first_day_week($day, $month, $year, $gm = false)
 {
@@ -688,8 +694,8 @@ function dol_get_first_day_week($day, $month, $year, $gm = false)
 	//print 'start_week='.$start_week.' tmparray[wday]='.$tmparray['wday'].' day offset='.$days.' seconds offset='.$seconds.'<br>';
 
 	//Get first day of week
-	$tmpdaytms = date($tmparray[0]) - $seconds; // $tmparray[0] is day of parameters
-	$tmpday = date("d", $tmpdaytms);
+	$tmpdaytms = (int) date((string) $tmparray['0']) - $seconds; // $tmparray[0] is day of parameters
+	$tmpday = idate("d", $tmpdaytms);
 
 	//Check first day of week is in same month than current day or not
 	if ($tmpday > $day) {
@@ -762,7 +768,7 @@ function getGMTEasterDatetime($year)
  */
 function num_public_holiday($timestampStart, $timestampEnd, $country_code = '', $lastday = 0, $includesaturday = -1, $includesunday = -1, $includefriday = -1, $includemonday = -1)
 {
-	global $db, $conf, $mysoc;
+	global $conf, $db, $mysoc;
 
 	$nbFerie = 0;
 
@@ -775,35 +781,24 @@ function num_public_holiday($timestampStart, $timestampEnd, $country_code = '',
 		$country_code = $mysoc->country_code;
 	}
 	if ($includemonday < 0) {
-		$includemonday = (isset($conf->global->MAIN_NON_WORKING_DAYS_INCLUDE_MONDAY) ? $conf->global->MAIN_NON_WORKING_DAYS_INCLUDE_MONDAY : 0);
+		$includemonday = getDolGlobalInt('MAIN_NON_WORKING_DAYS_INCLUDE_MONDAY', 0);
 	}
 	if ($includefriday < 0) {
-		$includefriday = (isset($conf->global->MAIN_NON_WORKING_DAYS_INCLUDE_FRIDAY) ? $conf->global->MAIN_NON_WORKING_DAYS_INCLUDE_FRIDAY : 0);
+		$includefriday = getDolGlobalInt('MAIN_NON_WORKING_DAYS_INCLUDE_FRIDAY', 0);
 	}
 	if ($includesaturday < 0) {
-		$includesaturday = (isset($conf->global->MAIN_NON_WORKING_DAYS_INCLUDE_SATURDAY) ? $conf->global->MAIN_NON_WORKING_DAYS_INCLUDE_SATURDAY : 1);
+		$includesaturday = getDolGlobalInt('MAIN_NON_WORKING_DAYS_INCLUDE_SATURDAY', 1);
 	}
 	if ($includesunday < 0) {
-		$includesunday = (isset($conf->global->MAIN_NON_WORKING_DAYS_INCLUDE_SUNDAY) ? $conf->global->MAIN_NON_WORKING_DAYS_INCLUDE_SUNDAY : 1);
+		$includesunday = getDolGlobalInt('MAIN_NON_WORKING_DAYS_INCLUDE_SUNDAY', 1);
 	}
 
 	$country_id = dol_getIdFromCode($db, $country_code, 'c_country', 'code', 'rowid');
 
-	$i = 0;
-	while ((($lastday == 0 && $timestampStart < $timestampEnd) || ($lastday && $timestampStart <= $timestampEnd))
-		&& ($i < 50000)) {		// Loop end when equals (Test on i is a security loop to avoid infinite loop)
-		$ferie = false;
-		$specialdayrule = array();
-
-		$jour  = gmdate("d", $timestampStart);
-		$mois  = gmdate("m", $timestampStart);
-		$annee = gmdate("Y", $timestampStart);
-
-		//print "jour=".$jour." month=".$mois." year=".$annee." includesaturday=".$includesaturday." includesunday=".$includesunday."\n";
-
+	if (empty($conf->cache['arrayOfActivePublicHolidays_'.$country_id])) {
 		// Loop on public holiday defined into hrm_public_holiday for the day, month and year analyzed
-		// TODO Execute this request first and store results into an array, then reuse this array.
-		$sql = "SELECT code, entity, fk_country, dayrule, year, month, day, active";
+		$tmpArrayOfPublicHolidays = array();
+		$sql = "SELECT id, code, entity, fk_country, dayrule, year, month, day, active";
 		$sql .= " FROM ".MAIN_DB_PREFIX."c_hrm_public_holiday";
 		$sql .= " WHERE active = 1 and fk_country IN (0".($country_id > 0 ? ", ".$country_id : 0).")";
 		$sql .= " AND entity IN (0," .getEntity('holiday') .")";
@@ -814,32 +809,53 @@ function num_public_holiday($timestampStart, $timestampEnd, $country_code = '',
 			$i = 0;
 			while ($i < $num_rows) {
 				$obj = $db->fetch_object($resql);
-
-				if (!empty($obj->dayrule) && $obj->dayrule != 'date') {		// For example 'easter', '...'
-					$specialdayrule[$obj->dayrule] = $obj->dayrule;
-				} else {
-					$match = 1;
-					if (!empty($obj->year) && $obj->year != $annee) {
-						$match = 0;
-					}
-					if ($obj->month != $mois) {
-						$match = 0;
-					}
-					if ($obj->day != $jour) {
-						$match = 0;
-					}
-
-					if ($match) {
-						$ferie = true;
-					}
-				}
-
+				$tmpArrayOfPublicHolidays[$obj->id] = array('dayrule' => $obj->dayrule, 'year' => $obj->year, 'month' => $obj->month, 'day' => $obj->day);
 				$i++;
 			}
 		} else {
 			dol_syslog($db->lasterror(), LOG_ERR);
 			return 'Error sql '.$db->lasterror();
 		}
+
+		//var_dump($tmpArrayOfPublicHolidays);
+		$conf->cache['arrayOfActivePublicHolidays_'.$country_id] = $tmpArrayOfPublicHolidays;
+	}
+
+	$arrayOfPublicHolidays = $conf->cache['arrayOfActivePublicHolidays_'.$country_id];
+
+	$i = 0;
+	while ((($lastday == 0 && $timestampStart < $timestampEnd) || ($lastday && $timestampStart <= $timestampEnd))
+		&& ($i < 50000)) {		// Loop end when equals (Test on i is a security loop to avoid infinite loop)
+		$ferie = false;
+		$specialdayrule = array();
+
+		$jour  = (int) gmdate("d", $timestampStart);
+		$mois  = (int) gmdate("m", $timestampStart);
+		$annee = (int) gmdate("Y", $timestampStart);
+
+		//print "jour=".$jour." month=".$mois." year=".$annee." includesaturday=".$includesaturday." includesunday=".$includesunday."\n";
+		foreach ($arrayOfPublicHolidays as $entrypublicholiday) {
+			if (!empty($entrypublicholiday['dayrule']) && $entrypublicholiday['dayrule'] != 'date') {		// For example 'easter', '...'
+				$specialdayrule[$entrypublicholiday['dayrule']] = $entrypublicholiday['dayrule'];
+			} else {
+				$match = 1;
+				if (!empty($entrypublicholiday['year']) && $entrypublicholiday['year'] != $annee) {
+					$match = 0;
+				}
+				if ($entrypublicholiday['month'] != $mois) {
+					$match = 0;
+				}
+				if ($entrypublicholiday['day'] != $jour) {
+					$match = 0;
+				}
+
+				if ($match) {
+					$ferie = true;
+				}
+			}
+
+			$i++;
+		}
 		//var_dump($specialdayrule)."\n";
 		//print "ferie=".$ferie."\n";
 
@@ -950,7 +966,7 @@ function num_public_holiday($timestampStart, $timestampEnd, $country_code = '',
 				$jour_1sunsept = date("d", $date_1sunsept);
 				$mois_1sunsept = date("m", $date_1sunsept);
 				if ($jour_1sunsept == $jour && $mois_1sunsept == $mois) {
-					$ferie=true;
+					$ferie = true;
 				}
 				// Geneva fast in Switzerland
 			}
@@ -1005,17 +1021,19 @@ function num_public_holiday($timestampStart, $timestampEnd, $country_code = '',
  *	@param	   int			$timestampEnd       Timestamp end UTC
  *	@param     int			$lastday            Last day is included, 0: no, 1:yes
  *	@return    int								Number of days
- *  @seealso num_public_holiday(), num_open_day()
+ *  @see num_public_holiday(), num_open_day()
  */
 function num_between_day($timestampStart, $timestampEnd, $lastday = 0)
 {
-	if ($timestampStart < $timestampEnd) {
+	if ($timestampStart <= $timestampEnd) {
 		if ($lastday == 1) {
 			$bit = 0;
 		} else {
 			$bit = 1;
 		}
 		$nbjours = (int) floor(($timestampEnd - $timestampStart) / (60 * 60 * 24)) + 1 - $bit;
+	} else {
+		$nbjours = 0;
 	}
 	//print ($timestampEnd - $timestampStart) - $lastday;
 	return $nbjours;
@@ -1058,7 +1076,7 @@ function num_open_day($timestampStart, $timestampEnd, $inhour = 0, $lastday = 0,
 		$numholidays = num_public_holiday($timestampStart, $timestampEnd, $country_code, $lastday);
 		$nbOpenDay = ($numdays - $numholidays);
 		if ($inhour == 1 && $nbOpenDay <= 3) {
-			$nbOpenDay = ($nbOpenDay * 24);
+			$nbOpenDay *= 24;
 		}
 		return $nbOpenDay - (($inhour == 1 ? 12 : 0.5) * abs($halfday));
 	} elseif ($timestampStart == $timestampEnd) {
@@ -1073,7 +1091,7 @@ function num_open_day($timestampStart, $timestampEnd, $inhour = 0, $lastday = 0,
 		$nbOpenDay = $lastday;
 
 		if ($inhour == 1) {
-			$nbOpenDay = ($nbOpenDay * 24);
+			$nbOpenDay *= 24;
 		}
 		return $nbOpenDay - (($inhour == 1 ? 12 : 0.5) * abs($halfday));
 	} else {
@@ -1089,7 +1107,7 @@ function num_open_day($timestampStart, $timestampEnd, $inhour = 0, $lastday = 0,
  *
  *	@param	Translate	$outputlangs	Object langs
  *  @param	int			$short			0=Return long label, 1=Return short label
- *	@return array						Month string or array if selected < 0
+ *	@return array<int<1,12>,string>		String of months in normal or short string format
  */
 function monthArray($outputlangs, $short = 0)
 {
@@ -1127,12 +1145,13 @@ function monthArray($outputlangs, $short = 0)
 
 	return $montharray;
 }
+
 /**
  *	Return array of week numbers.
  *
  *	@param	int 		$month			Month number
  *  @param	int			$year			Year number
- *	@return array						Week numbers
+ *	@return string[]					Week numbers (week 1 is '01')
  */
 function getWeekNumbersOfMonth($month, $year)
 {
@@ -1144,12 +1163,13 @@ function getWeekNumbersOfMonth($month, $year)
 	}
 	return $TWeek;
 }
+
 /**
  *	Return array of first day of weeks.
  *
- *	@param	array 		$TWeek			array of week numbers
+ *	@param	string[] 	$TWeek			array of week numbers we want (week 1 must be '01')
  *  @param	int			$year			Year number
- *	@return array						First day of week
+ *	@return string[]					First day of each week in entry (day 1 is '01')
  */
 function getFirstDayOfEachWeek($TWeek, $year)
 {
@@ -1162,12 +1182,13 @@ function getFirstDayOfEachWeek($TWeek, $year)
 	}
 	return $TFirstDayOfWeek;
 }
+
 /**
  *	Return array of last day of weeks.
  *
- *	@param	array 		$TWeek			array of week numbers
+ *	@param	string[] 	$TWeek			array of week numbers
  *  @param	int			$year			Year number
- *	@return array						Last day of week
+ *	@return string[]					Last day of week (day 1 is '01')
  */
 function getLastDayOfEachWeek($TWeek, $year)
 {
@@ -1177,13 +1198,14 @@ function getLastDayOfEachWeek($TWeek, $year)
 	}
 	return $TLastDayOfWeek;
 }
+
 /**
  *	Return week number.
  *
  *	@param	int 		$day			Day number
  *	@param	int 		$month			Month number
  *  @param	int			$year			Year number
- *	@return int							Week number
+ *	@return string						Week number as two digits (week 1 is '01')
  */
 function getWeekNumber($day, $month, $year)
 {
