@@ -4,7 +4,8 @@
  * Copyright (C) 2015       Florian Henry       <florian.henry@open-concept.pro>
  * Copyright (C) 2015       Raphaël Doursenaud  <rdoursenaud@gpcsolutions.fr>
  * Copyright (C) 2018       Francis Appels      <francis.appels@yahoo.com>
- * Copyright (C) 2019       Frédéric France     <frederic.france@netlogic.fr>
+ * Copyright (C) 2019-2024  Frédéric France     <frederic.france@free.fr>
+ * Copyright (C) 2024		MDW					<mdeweerd@users.noreply.github.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +29,7 @@
 
 // Put here all includes required by your class file
 require_once DOL_DOCUMENT_ROOT.'/core/class/commonobject.class.php';
+require_once DOL_DOCUMENT_ROOT.'/core/class/commonobjectline.class.php';
 
 /**
  * Class to manage ECM files
@@ -95,6 +97,11 @@ class EcmFiles extends CommonObject
 	 */
 	public $keywords;
 
+	/**
+	 * @var ?string content
+	 */
+	public $content;
+
 	/**
 	 * @var string cover
 	 */
@@ -106,7 +113,7 @@ class EcmFiles extends CommonObject
 	public $position;
 
 	/**
-	 * @var string can be 'generated', 'uploaded', 'unknown'
+	 * @var 'generated'|'uploaded'|'unknown'|'copy'|''
 	 */
 	public $gen_or_uploaded;
 
@@ -116,12 +123,12 @@ class EcmFiles extends CommonObject
 	public $extraparams;
 
 	/**
-	 * @var int|string date create
+	 * @var int|'' date create
 	 */
 	public $date_c = '';
 
 	/**
-	 * @var int|string date modify
+	 * @var int|'' date modify
 	 */
 	public $date_m = '';
 
@@ -155,12 +162,38 @@ class EcmFiles extends CommonObject
 	 */
 	public $section_id;
 
+	public $fields = array(
+		'rowid' => array('type' => 'integer', 'label' => 'TechnicalID', 'enabled' => 1, 'position' => 1, 'notnull' => 1, 'visible' => 0, 'noteditable' => 1, 'index' => 1, 'css' => 'left', 'comment' => "Id"),
+		'ref' => array('type' => 'varchar(128)', 'label' => 'Ref', 'enabled' => 1, 'position' => 20, 'notnull' => 1, 'visible' => -1, 'index' => 1, 'searchall' => 1, 'showoncombobox' => 1, 'validate' => 1, 'comment' => "contains hash from filename+filepath"),
+		'label' => array('type' => 'varchar(128)', 'label' => 'Label', 'enabled' => 1, 'position' => 30, 'notnull' => 0, 'visible' => -1, 'searchall' => 1, 'css' => 'minwidth300', 'cssview' => 'wordbreak', 'showoncombobox' => 2, 'validate' => 1, 'comment' => "contains hash of file content"),
+		'share' => array('type' => 'varchar(128)', 'label' => 'Share', 'enabled' => 1, 'position' => 40, 'notnull' => 0, 'visible' => -1, 'searchall' => 1, 'css' => 'minwidth300', 'cssview' => 'wordbreak', 'showoncombobox' => 2, 'validate' => 1, 'comment' => "contains hash for file sharing"),
+		'entity' => array('type' => 'integer', 'label' => 'Entity', 'default' => '1', 'enabled' => 1, 'visible' => -2, 'notnull' => -1, 'position' => 50, 'index' => 1),
+		'filepath' => array('type' => 'varchar(255)', 'label' => 'FilePath', 'enabled' => 1, 'position' => 60, 'notnull' => 0, 'visible' => 0, 'searchall' => 0, 'css' => 'minwidth300', 'cssview' => 'wordbreak', 'showoncombobox' => 2, 'validate' => 1,'comment' => "relative to dolibarr document dir. Example module/def"),
+		'filename' => array('type' => 'varchar(255)', 'label' => 'FileName', 'enabled' => 1, 'position' => 70, 'notnull' => 0, 'visible' => 1, 'searchall' => 1, 'css' => 'minwidth300', 'cssview' => 'wordbreak', 'showoncombobox' => 2, 'validate' => 1,'comment' => "file name only without any directory"),
+		'src_object_type' => array('type' => 'varchar(64)', 'label' => 'SourceType', 'enabled' => 1, 'position' => 80, 'notnull' => 0, 'visible' => 0, 'searchall' => 1, 'css' => 'minwidth300', 'cssview' => 'wordbreak', 'showoncombobox' => 2, 'validate' => 1,'comment' => "Source object type ('proposal', 'invoice', ...)"),
+		'src_object_id' => array('type' => 'integer', 'label' => 'SourceID', 'default' => '1', 'enabled' => 1, 'visible' => 0, 'notnull' => 1, 'position' => 90, 'index' => 1, 'comment' => "Source object id"),
+		'fullpath_orig' => array('type' => 'varchar(750)', 'label' => 'FullPathOrig', 'enabled' => 1, 'position' => 100, 'notnull' => 0, 'visible' => 0, 'searchall' => 0, 'css' => 'minwidth300', 'cssview' => 'wordbreak', 'showoncombobox' => 2, 'validate' => 1,'comment' => "full path of original filename, when file is uploaded from a local computer"),
+		'description' => array('type' => 'text', 'label' => 'Description', 'enabled' => 1, 'visible' => 0, 'position' => 110),
+		'keywords' => array('type' => 'varchar(750)', 'label' => 'Keywords', 'enabled' => 1, 'position' => 120, 'notnull' => 0, 'visible' => 1, 'searchall' => 1, 'css' => 'minwidth300', 'cssview' => 'wordbreak', 'showoncombobox' => 2, 'validate' => 1,'comment' => "list of keywords, separated with comma. Must be limited to most important keywords."),
+		'content' => array('type' => 'html', 'label' => 'Content', 'enabled' => 'getDolGlobalString("MAIN_SAVE_FILE_CONTENT_AS_TEXT")', 'position' => 120, 'notnull' => 0, 'visible' => 1, 'searchall' => 1, 'css' => 'minwidth300', 'cssview' => 'wordbreak', 'csslist' => 'tdoverflowmax200', 'help' => "Text content of file", 'showoncombobox' => 2, 'validate' => 1,'comment' => "Text content if option to store txt content was set."),
+		'cover' => array('type' => 'text', 'label' => 'Cover', 'enabled' => 1, 'visible' => 0, 'position' => 130, 'comment' => "is this file a file to use for a cover"),
+		'position' => array('type' => 'integer', 'label' => 'Position', 'default' => '1', 'enabled' => 1, 'visible' => -2, 'notnull' => 1, 'position' => 140, 'index' => 1, 'comment' => "position of file among others"),
+		'gen_or_uploaded' => array('type' => 'varchar(12)', 'label' => 'GenOrUpload', 'enabled' => 1, 'position' => 150, 'notnull' => 0, 'visible' => -1, 'searchall' => 1, 'css' => 'minwidth300', 'cssview' => 'wordbreak', 'showoncombobox' => 2, 'validate' => 1,'comment' => "'generated' or 'uploaded'"),
+		'extraparams' => array('type' => 'varchar(255)', 'label' => 'ExtraParams', 'enabled' => 1, 'position' => 160, 'notnull' => 0, 'visible' => 1, 'searchall' => 1, 'css' => 'minwidth300', 'cssview' => 'wordbreak', 'showoncombobox' => 2, 'validate' => 1, 'comment' => "for stocking other parameters with json format"),
+		'date_c' => array('type' => 'datetime', 'label' => 'DateCreation', 'enabled' => 1, 'visible' => -1, 'position' => 170),
+		'tms' => array('type' => 'timestamp', 'label' => 'DateModification', 'enabled' => 1, 'visible' => -1, 'notnull' => 1, 'position' => 175),
+		'fk_user_c' => array('type' => 'integer:User:user/class/user.class.php', 'label' => 'UserAuthor', 'enabled' => 1, 'position' => 510, 'notnull' => 1, 'visible' => -2, 'foreignkey' => 'user.rowid',),
+		'fk_user_m' => array('type' => 'integer:User:user/class/user.class.php', 'label' => 'UserModif', 'enabled' => 1, 'position' => 511, 'notnull' => -1, 'visible' => -2,),
+		'note_public' => array('type' => 'text', 'label' => 'NotePublic', 'enabled' => 1, 'visible' => 0, 'position' => 155),
+		'note_private' => array('type' => 'text', 'label' => 'NotePrivate', 'enabled' => 1, 'visible' => 0, 'position' => 160),
+		'acl' => array('type' => 'text', 'label' => 'NotePrivate', 'enabled' => 1, 'visible' => 0, 'position' => 160, 'comment' => "for future permission 'per file'"),
+	);
 
 
 	/**
 	 * Constructor
 	 *
-	 * @param DoliDb $db Database handler
+	 * @param DoliDB $db Database handler
 	 */
 	public function __construct(DoliDB $db)
 	{
@@ -171,10 +204,10 @@ class EcmFiles extends CommonObject
 	 * Create object into database
 	 *
 	 * @param  User $user      	User that creates
-	 * @param  bool $notrigger 	false=launch triggers after, true=disable triggers
-	 * @return int 				<0 if KO, Id of created object if OK
+	 * @param  int 	$notrigger 	0=launch triggers after, 1=disable triggers
+	 * @return int 				Return integer <0 if KO, Id of created object if OK
 	 */
-	public function create(User $user, $notrigger = false)
+	public function create(User $user, $notrigger = 0)
 	{
 		global $conf;
 
@@ -242,7 +275,7 @@ class EcmFiles extends CommonObject
 		// If ref not defined
 		if (empty($this->ref)) {
 			include_once DOL_DOCUMENT_ROOT.'/core/lib/security.lib.php';
-			$this->ref = dol_hash($this->filepath.'/'.$this->filename, 3);
+			$this->ref = dol_hash($this->filepath.'/'.$this->filename, '3');
 		}
 
 		$maxposition = 0;
@@ -259,7 +292,7 @@ class EcmFiles extends CommonObject
 				$this->errors[] = 'Error '.$this->db->lasterror();
 				return --$error;
 			}
-			$maxposition = $maxposition + 1;
+			$maxposition += 1;
 		} else {
 			$maxposition = $this->position;
 		}
@@ -285,6 +318,9 @@ class EcmFiles extends CommonObject
 		$sql .= 'fullpath_orig,';
 		$sql .= 'description,';
 		$sql .= 'keywords,';
+		if (getDolGlobalString("MAIN_SAVE_FILE_CONTENT_AS_TEXT")) {
+			$sql .= 'content,';
+		}
 		$sql .= 'cover,';
 		$sql .= 'position,';
 		$sql .= 'gen_or_uploaded,';
@@ -306,12 +342,15 @@ class EcmFiles extends CommonObject
 		$sql .= ' '.(!isset($this->fullpath_orig) ? 'NULL' : "'".$this->db->escape($this->fullpath_orig)."'").',';
 		$sql .= ' '.(!isset($this->description) ? 'NULL' : "'".$this->db->escape($this->description)."'").',';
 		$sql .= ' '.(!isset($this->keywords) ? 'NULL' : "'".$this->db->escape($this->keywords)."'").',';
+		if (getDolGlobalString("MAIN_SAVE_FILE_CONTENT_AS_TEXT")) {
+			$sql .= ' '.(!isset($this->content) ? 'NULL' : "'".$this->db->escape($this->content)."'").',';
+		}
 		$sql .= ' '.(!isset($this->cover) ? 'NULL' : "'".$this->db->escape($this->cover)."'").',';
 		$sql .= ' '.((int) $maxposition).',';
 		$sql .= ' '.(!isset($this->gen_or_uploaded) ? 'NULL' : "'".$this->db->escape($this->gen_or_uploaded)."'").',';
 		$sql .= ' '.(!isset($this->extraparams) ? 'NULL' : "'".$this->db->escape($this->extraparams)."'").',';
 		$sql .= " '".$this->db->idate($this->date_c)."',";
-		$sql .= ' '.(!isset($this->date_m) || dol_strlen($this->date_m) == 0 ? 'NULL' : "'".$this->db->idate($this->date_m)."'").',';
+		$sql .= ' '.(!isset($this->date_m) || dol_strlen((string) $this->date_m) == 0 ? 'NULL' : "'".$this->db->idate($this->date_m)."'").',';
 		$sql .= ' '.(!isset($this->fk_user_c) ? $user->id : $this->fk_user_c).',';
 		$sql .= ' '.(!isset($this->fk_user_m) ? 'NULL' : $this->fk_user_m).',';
 		$sql .= ' '.(!isset($this->acl) ? 'NULL' : "'".$this->db->escape($this->acl)."'").',';
@@ -366,10 +405,10 @@ class EcmFiles extends CommonObject
 	 * @param  string $ref         	   	Hash of file name (filename+filepath). Not always defined on some version.
 	 * @param  string $relativepath    	Relative path of file from document directory. Example: 'path/path2/file' or 'path/path2/*'
 	 * @param  string $hashoffile      	Hash of file content. Take the first one found if same file is at different places. This hash will also change if file content is changed.
-	 * @param  string $hashforshare    	Hash of file sharing or 'shared'
+	 * @param  string $hashforshare    	Hash of file sharing, or 'shared'
 	 * @param  string $src_object_type 	src_object_type to search (value of object->table_element)
-	 * @param  string $src_object_id 	src_object_id to search
-	 * @return int                 	   	<0 if KO, 0 if not found, >0 if OK
+	 * @param  int    $src_object_id 	src_object_id to search
+	 * @return int                 	   	Return integer <0 if KO, 0 if not found, >0 if OK
 	 */
 	public function fetch($id, $ref = '', $relativepath = '', $hashoffile = '', $hashforshare = '', $src_object_type = '', $src_object_id = 0)
 	{
@@ -388,6 +427,9 @@ class EcmFiles extends CommonObject
 		$sql .= " t.fullpath_orig,";
 		$sql .= " t.description,";
 		$sql .= " t.keywords,";
+		if (getDolGlobalString("MAIN_SAVE_FILE_CONTENT_AS_TEXT")) {
+			$sql .= " t.content,";
+		}
 		$sql .= " t.cover,";
 		$sql .= " t.position,";
 		$sql .= " t.gen_or_uploaded,";
@@ -407,6 +449,7 @@ class EcmFiles extends CommonObject
 		 if (isModEnabled('multicompany')) {
 		 $sql .= " AND entity IN (" . getEntity('ecmfiles') . ")";
 		 }*/
+		$filterfound = 0;
 		if ($relativepath) {
 			$relativepathwithnoexe = preg_replace('/\.noexe$/', '', $relativepath); // We must never have the .noexe into the database
 			$sql .= " AND t.filepath = '".$this->db->escape(dirname($relativepath))."'";
@@ -415,28 +458,38 @@ class EcmFiles extends CommonObject
 				$sql .= " AND t.filename = '".$this->db->escape($filename)."'";
 			}
 			$sql .= " AND t.entity = ".$conf->entity; // unique key include the entity so each company has its own index
-		} elseif (!empty($ref)) {		// hash of file path
+			$filterfound++;
+		}
+		if (!empty($ref)) {		// hash of file path
 			$sql .= " AND t.ref = '".$this->db->escape($ref)."'";
 			$sql .= " AND t.entity = ".$conf->entity; // unique key include the entity so each company has its own index
-		} elseif (!empty($hashoffile)) {	// hash of content
+			$filterfound++;
+		}
+		if (!empty($hashoffile)) {	// hash of content
 			$sql .= " AND t.label = '".$this->db->escape($hashoffile)."'";
 			$sql .= " AND t.entity = ".$conf->entity; // unique key include the entity so each company has its own index
-		} elseif (!empty($hashforshare)) {
+			$filterfound++;
+		}
+		if (!empty($hashforshare)) {
 			if ($hashforshare != 'shared') {
 				$sql .= " AND t.share = '".$this->db->escape($hashforshare)."'";
 			} else {
 				$sql .= " AND t.share IS NOT NULL AND t.share <> ''";
 			}
 			//$sql .= " AND t.entity = ".$conf->entity;							// hashforshare already unique
-		} elseif ($src_object_type && $src_object_id) {
-			// Warning: May return several record, and only first one is returned !
+			$filterfound++;
+		}
+		if ($src_object_type && $src_object_id) {
 			$sql .= " AND t.src_object_type = '".$this->db->escape($src_object_type)."' AND t.src_object_id = ".((int) $src_object_id);
 			$sql .= " AND t.entity = ".((int) $conf->entity);
-		} else {
+			$filterfound++;
+		}
+		if ($id > 0 || empty($filterfound)) {
 			$sql .= ' AND t.rowid = '.((int) $id); // rowid already unique
 		}
 
-		$this->db->plimit(1); // When we search on src or on hash of content (hashforfile) to solve hash conflict when several files has same content, we take first one only
+		// Warning: May return several record, and only first one is returned !
+		$this->db->plimit(1); // When we search on src, or on hash of content (hashforfile), we take first one only
 		$this->db->order('t.rowid', 'ASC');
 
 		$resql = $this->db->query($sql);
@@ -455,6 +508,9 @@ class EcmFiles extends CommonObject
 				$this->fullpath_orig = $obj->fullpath_orig;
 				$this->description = $obj->description;
 				$this->keywords = $obj->keywords;
+				if (getDolGlobalString("MAIN_SAVE_FILE_CONTENT_AS_TEXT")) {
+					$this->content = $obj->content;
+				}
 				$this->cover = $obj->cover;
 				$this->position = $obj->position;
 				$this->gen_or_uploaded = $obj->gen_or_uploaded;
@@ -494,16 +550,15 @@ class EcmFiles extends CommonObject
 	/**
 	 * Load object in memory from the database
 	 *
-	 * @param string $sortorder Sort Order
-	 * @param string $sortfield Sort field
-	 * @param int    $limit     offset limit
-	 * @param int    $offset    offset limit
-	 * @param array  $filter    filter array
-	 * @param string $filtermode filter mode (AND or OR)
-	 *
-	 * @return int <0 if KO, >0 if OK
+	 * @param 	string 			$sortorder 		Sort Order
+	 * @param 	string 			$sortfield 		Sort field
+	 * @param 	int    			$limit     		Limit
+	 * @param 	int    			$offset    		Offset limit
+	 * @param 	string|array<string,mixed> 	$filter		filter array
+	 * @param 	string 			$filtermode 	filter mode (AND or OR)
+	 * @return 	int 							Return integer <0 if KO, >0 if OK
 	 */
-	public function fetchAll($sortorder = '', $sortfield = '', $limit = 0, $offset = 0, array $filter = array(), $filtermode = 'AND')
+	public function fetchAll($sortorder = '', $sortfield = '', $limit = 0, $offset = 0, $filter = '', $filtermode = 'AND')
 	{
 		dol_syslog(__METHOD__, LOG_DEBUG);
 
@@ -517,6 +572,9 @@ class EcmFiles extends CommonObject
 		$sql .= " t.fullpath_orig,";
 		$sql .= " t.description,";
 		$sql .= " t.keywords,";
+		if (getDolGlobalString("MAIN_SAVE_FILE_CONTENT_AS_TEXT")) {
+			$sql .= " t.content,";
+		}
 		$sql .= " t.cover,";
 		$sql .= " t.position,";
 		$sql .= " t.gen_or_uploaded,";
@@ -529,26 +587,40 @@ class EcmFiles extends CommonObject
 		$sql .= " t.src_object_type,";
 		$sql .= " t.src_object_id";
 		$sql .= ' FROM '.MAIN_DB_PREFIX.$this->table_element.' as t';
+		$sql .= ' WHERE 1 = 1';
 
 		// Manage filter
-		$sqlwhere = array();
-		if (count($filter) > 0) {
-			foreach ($filter as $key => $value) {
-				if ($key == 't.src_object_id') {
-					$sqlwhere[] = $key." = ".((int) $value);
-				} else {
-					$sqlwhere[] = $key." LIKE '%".$this->db->escape($value)."%'";
+		if (is_array($filter)) {
+			$sqlwhere = array();
+			if (count($filter) > 0) {
+				foreach ($filter as $key => $value) {
+					if ($key == 't.src_object_id') {
+						$sqlwhere[] = $this->db->sanitize($key)." = ".((int) $value);
+					} else {
+						$sqlwhere[] = $this->db->sanitize($key)." LIKE '%".$this->db->escape($this->db->escapeforlike($value))."%'";
+					}
 				}
 			}
+			if (count($sqlwhere) > 0) {
+				$sql .= ' AND '.implode(' '.$this->db->escape($filtermode).' ', $sqlwhere);
+			}
+
+			$filter = '';
 		}
-		$sql .= ' WHERE 1 = 1';
+
+		// Manage filter
+		$errormessage = '';
+		$sql .= forgeSQLFromUniversalSearchCriteria($filter, $errormessage);
+		if ($errormessage) {
+			$this->errors[] = $errormessage;
+			dol_syslog(__METHOD__.' '.implode(',', $this->errors), LOG_ERR);
+			return -1;
+		}
+
 		/* Fetching this table depends on filepath+filename, it must not depends on entity
 		 if (isModEnabled('multicompany')) {
 		 $sql .= " AND entity IN (" . getEntity('ecmfiles') . ")";
 		 }*/
-		if (count($sqlwhere) > 0) {
-			$sql .= ' AND '.implode(' '.$this->db->escape($filtermode).' ', $sqlwhere);
-		}
 		if (!empty($sortfield)) {
 			$sql .= $this->db->order($sortfield, $sortorder);
 		}
@@ -563,7 +635,7 @@ class EcmFiles extends CommonObject
 			$num = $this->db->num_rows($resql);
 
 			while ($obj = $this->db->fetch_object($resql)) {
-				$line = new EcmfilesLine();
+				$line = new EcmFilesLine($this->db);
 
 				$line->id = $obj->rowid;
 				$line->ref = $obj->rowid;
@@ -575,6 +647,9 @@ class EcmFiles extends CommonObject
 				$line->fullpath_orig = $obj->fullpath_orig;
 				$line->description = $obj->description;
 				$line->keywords = $obj->keywords;
+				if (getDolGlobalString("MAIN_SAVE_FILE_CONTENT_AS_TEXT")) {
+					$line->content = $obj->content;
+				}
 				$line->cover = $obj->cover;
 				$line->position = $obj->position;
 				$line->gen_or_uploaded = $obj->gen_or_uploaded;
@@ -602,12 +677,11 @@ class EcmFiles extends CommonObject
 	/**
 	 * Update object into database
 	 *
-	 * @param  User $user      User that modifies
-	 * @param  bool $notrigger false=launch triggers after, true=disable triggers
-	 *
-	 * @return int <0 if KO, >0 if OK
+	 * @param  User $user      	User that modifies
+	 * @param  int 	$notrigger 	0=launch triggers after, 1=disable triggers
+	 * @return int 				Return integer <0 if KO, >0 if OK
 	 */
-	public function update(User $user, $notrigger = false)
+	public function update(User $user, $notrigger = 0)
 	{
 		global $conf;
 
@@ -627,7 +701,7 @@ class EcmFiles extends CommonObject
 			$this->share = trim($this->share);
 		}
 		if (isset($this->entity)) {
-			$this->entity = trim($this->entity);
+			$this->entity = (int) $this->entity;
 		}
 		if (isset($this->filename)) {
 			$this->filename = preg_replace('/\.noexe$/', '', trim($this->filename));
@@ -655,7 +729,7 @@ class EcmFiles extends CommonObject
 			$this->extraparams = trim($this->extraparams);
 		}
 		if (isset($this->fk_user_m)) {
-			$this->fk_user_m = trim($this->fk_user_m);
+			$this->fk_user_m = (int) $this->fk_user_m;
 		}
 		if (isset($this->acl)) {
 			$this->acl = trim($this->acl);
@@ -669,7 +743,7 @@ class EcmFiles extends CommonObject
 
 		// Update request
 		$sql = 'UPDATE '.MAIN_DB_PREFIX.$this->table_element.' SET';
-		$sql .= " ref = '".$this->db->escape(dol_hash($this->filepath."/".$this->filename, 3))."',";
+		$sql .= " ref = '".$this->db->escape(dol_hash($this->filepath."/".$this->filename, '3'))."',";
 		$sql .= ' label = '.(isset($this->label) ? "'".$this->db->escape($this->label)."'" : "null").',';
 		$sql .= ' share = '.(!empty($this->share) ? "'".$this->db->escape($this->share)."'" : "null").',';
 		$sql .= ' entity = '.(isset($this->entity) ? $this->entity : $conf->entity).',';
@@ -678,12 +752,15 @@ class EcmFiles extends CommonObject
 		$sql .= ' fullpath_orig = '.(isset($this->fullpath_orig) ? "'".$this->db->escape($this->fullpath_orig)."'" : "null").',';
 		$sql .= ' description = '.(isset($this->description) ? "'".$this->db->escape($this->description)."'" : "null").',';
 		$sql .= ' keywords = '.(isset($this->keywords) ? "'".$this->db->escape($this->keywords)."'" : "null").',';
+		if (getDolGlobalString("MAIN_SAVE_FILE_CONTENT_AS_TEXT")) {
+			$sql .= ' content = '.(isset($this->content) ? "'".$this->db->escape($this->content)."'" : "null").',';
+		}
 		$sql .= ' cover = '.(isset($this->cover) ? "'".$this->db->escape($this->cover)."'" : "null").',';
 		$sql .= ' position = '.(isset($this->position) ? $this->db->escape($this->position) : "0").',';
 		$sql .= ' gen_or_uploaded = '.(isset($this->gen_or_uploaded) ? "'".$this->db->escape($this->gen_or_uploaded)."'" : "null").',';
 		$sql .= ' extraparams = '.(isset($this->extraparams) ? "'".$this->db->escape($this->extraparams)."'" : "null").',';
 		$sql .= ' date_c = '.(!isset($this->date_c) || dol_strlen($this->date_c) != 0 ? "'".$this->db->idate($this->date_c)."'" : 'null').',';
-		//$sql .= ' tms = '.(! isset($this->date_m) || dol_strlen($this->date_m) != 0 ? "'".$this->db->idate($this->date_m)."'" : 'null').','; // Field automatically updated
+		//$sql .= ' tms = '.(! isset($this->date_m) || dol_strlen((string) $this->date_m) != 0 ? "'".$this->db->idate($this->date_m)."'" : 'null').','; // Field automatically updated
 		$sql .= ' fk_user_m = '.($this->fk_user_m > 0 ? $this->fk_user_m : $user->id).',';
 		$sql .= ' acl = '.(isset($this->acl) ? "'".$this->db->escape($this->acl)."'" : "null").',';
 		$sql .= ' src_object_id = '.($this->src_object_id > 0 ? $this->src_object_id : "null").',';
@@ -699,6 +776,14 @@ class EcmFiles extends CommonObject
 			dol_syslog(__METHOD__.' '.implode(',', $this->errors), LOG_ERR);
 		}
 
+		if (!$error) {
+			// Update extrafields
+			$result = $this->insertExtraFields();
+			if ($result < 0) {
+				$error++;
+			}
+		}
+
 		// Triggers
 		if (!$error && !$notrigger) {
 			// Call triggers
@@ -724,12 +809,11 @@ class EcmFiles extends CommonObject
 	/**
 	 * Delete object in database
 	 *
-	 * @param User $user      User that deletes
-	 * @param bool $notrigger false=launch triggers after, true=disable triggers
-	 *
-	 * @return int <0 if KO, >0 if OK
+	 * @param User 	$user      	User that deletes
+	 * @param int 	$notrigger 	0=launch triggers after, 1=disable triggers
+	 * @return int 				Return integer <0 if KO, >0 if OK
 	 */
-	public function delete(User $user, $notrigger = false)
+	public function delete(User $user, $notrigger = 0)
 	{
 		dol_syslog(__METHOD__, LOG_DEBUG);
 
@@ -791,7 +875,7 @@ class EcmFiles extends CommonObject
 		dol_syslog(__METHOD__, LOG_DEBUG);
 
 		$error = 0;
-		$object = new Ecmfiles($this->db);
+		$object = new EcmFiles($this->db);
 
 		$this->db->begin();
 
@@ -829,10 +913,69 @@ class EcmFiles extends CommonObject
 	}
 
 	/**
-	 *  Return a link to the object card (with optionaly the picto)
+	 * updateAfterRename update entries in ecmfiles if exist to avoid losing info
+	 *
+	 * @param  string $olddir old directory
+	 * @param  string $newdir new directory
+	 * @return void
+	 */
+	public function updateAfterRename($olddir, $newdir)
+	{
+		$sql = 'UPDATE '.MAIN_DB_PREFIX.'ecm_files SET';
+		$sql .= ' filepath = "'.$this->db->escape($newdir).'"';
+		//$sql .= ', fullpath_orig = "'.$dbs->escape($newdir)."'";
+		$sql .= ' WHERE ';
+		$sql .= ' filepath = "'.$this->db->escape($olddir).'"';
+		// $sql .= ' AND fullpath_orig = "'.$dbs->escape($olddir).'"';
+
+		$this->db->query($sql);
+	}
+
+	/**
+	 * getTooltipContentArray
+	 *
+	 * @param 	array<string,mixed> 	$params 		params to construct tooltip data
+	 * @return 	array{picto?:string,ref?:string,gen_or_upload?:string}|array{optimize:string}
+	 * @since v21
+	 */
+	public function getTooltipContentArray($params)
+	{
+		global $langs;
+
+		$langs->load('ecm');
+		$datas = [];
+		$nofetch = !empty($params['nofetch']);
+
+		if (getDolGlobalString('MAIN_OPTIMIZEFORTEXTBROWSER')) {
+			return ['optimize' => $langs->trans("ShowFile")];
+		}
+		$datas['picto'] = img_picto('', $this->picto, '', 0, 0, 0, '', 'paddingrightonly') . '<u>' . $langs->trans("File") . '</u>';
+		if (!empty($this->filename)) {
+			$datas['name'] = '<br><b>'.$langs->trans('Name').':</b> '.basename($this->filename);
+		}
+		if (!empty($this->ref)) {
+			$datas['ref'] = '<br><b>'.$langs->trans('HashOfFileContent').':</b> '.$this->ref;
+		}
+		if (!empty($this->share)) {
+			$datas['share'] = '<br>'.$langs->trans("FileSharedViaALink");
+		} else {
+			$datas['share'] = '<br>'.$langs->trans("FileNotShared");
+		}
+		if (!empty($this->gen_or_uploaded)) {
+			$datas['gen_or_upload'] = '<br><b>'.$langs->trans('GenOrUpload').':</b> '.$this->gen_or_uploaded;
+		}
+		if (!empty($this->content)) {
+			$datas['content'] = '<br>'.$langs->trans('FileHasAnIndexedTextContent');
+		}
+
+		return $datas;
+	}
+
+	/**
+	 *  Return a link to the object card (with optionally the picto)
 	 *
 	 *	@param	int		$withpicto			Include picto in link (0=No picto, 1=Include picto into link, 2=Only picto)
-	 *	@param	string	$option				On what the link point to
+	 *	@param	string	$option				On what the link point to (propal, etc) module name
 	 *  @param	int  	$notooltip			1=Disable tooltip
 	 *  @param	int		$maxlen				Max length of visible user name
 	 *  @param  string  $morecss            Add more css on link
@@ -840,9 +983,7 @@ class EcmFiles extends CommonObject
 	 */
 	public function getNomUrl($withpicto = 0, $option = '', $notooltip = 0, $maxlen = 24, $morecss = '')
 	{
-		global $db, $conf, $langs;
-		global $dolibarr_main_authentication, $dolibarr_main_demo;
-		global $menumanager, $hookmanager;
+		global $conf, $hookmanager, $langs;
 
 		if (!empty($conf->dol_no_mouse_hover)) {
 			$notooltip = 1; // Force disable tooltips
@@ -850,52 +991,93 @@ class EcmFiles extends CommonObject
 
 		$result = '';
 
-		$label = '<u>'.$langs->trans("MyModule").'</u>';
-		$label .= '<br>';
-		$label .= '<b>'.$langs->trans('Ref').':</b> '.$this->ref;
+		$params = [
+			'id' => $this->id,
+			'objecttype' => $this->element,
+			'option' => $option,
+			'nofetch' => 1,
+		];
+		$classfortooltip = 'classfortooltip';
+		$dataparams = '';
+		if (getDolGlobalInt('MAIN_ENABLE_AJAX_TOOLTIP')) {
+			$classfortooltip = 'classforajaxtooltip';
+			$dataparams = ' data-params="'.dol_escape_htmltag(json_encode($params)).'"';
+			$label = '';
+		} else {
+			$label = implode($this->getTooltipContentArray($params));
+		}
 
-		$url = DOL_URL_ROOT.'/ecm/'.$this->table_name.'_card.php?id='.$this->id;
+		if ($option) {
+			if ($option == 'facture_fournisseur') {
+				$tmppath = preg_replace('/^(\d+\/)?fournisseur\/facture\//', '', $this->filepath);
+			} elseif ($option == 'commande_fournisseur') {
+				$tmppath = preg_replace('/^(\d+\/)?fournisseur\/commande\//', '', $this->filepath);
+			} elseif ($option == 'tax-vat') {	// Remove part "tax/vat/"
+				$tmppath = preg_replace('/^(\d+\/)?tax\/vat\//', '', $this->filepath);
+			} elseif ($option == 'remisecheque') {	// Remove part "tax/vat/"
+				$tmppath = preg_replace('/^bank\/checkdeposits\//', '', $this->filepath);
+			} else {
+				if ((int) $this->entity > 1) {
+					// Remove the part "entityid/commande/" into "entityid/commande/REFXXX" to get only the ref
+					$tmppath = preg_replace('/^\d+\/[^\/]+\//', '', $this->filepath);
+				} else {
+					// Remove the part "commande/" into "commande/REFXXX" to get only the ref
+					$tmppath = preg_replace('/^[^\/]+\//', '', $this->filepath);
+				}
+			}
+			$url = DOL_URL_ROOT.'/document.php?modulepart='.urlencode($option).'&file='.urlencode($tmppath.'/'.$this->filename).'&entity='.((int) $this->entity);
+		} else {
+			$url = DOL_URL_ROOT.'/ecm/file_card.php?id='.$this->id;
+		}
 
 		$linkclose = '';
 		if (empty($notooltip)) {
-			if (!empty($conf->global->MAIN_OPTIMIZEFORTEXTBROWSER)) {
-				$label = $langs->trans("ShowProject");
-				$linkclose .= ' alt="'.dol_escape_htmltag($label, 1).'"';
+			if (getDolGlobalString('MAIN_OPTIMIZEFORTEXTBROWSER')) {
+				$label = $langs->trans("ShowFile");
+				$linkclose .= ' alt="'.dolPrintHTMLForAttribute($label).'"';
 			}
-			$linkclose .= ' title="'.dol_escape_htmltag($label, 1).'"';
-			$linkclose .= ' class="classfortooltip'.($morecss ? ' '.$morecss : '').'"';
+			$linkclose .= ($label ? ' title="'.dolPrintHTMLForAttribute($label).'"' : ' title="tocomplete"');
+			$linkclose .= $dataparams.' class="'.$classfortooltip.' '.($morecss ? ' '.$morecss : '').'"';
 		} else {
 			$linkclose = ($morecss ? ' class="'.$morecss.'"' : '');
 		}
 
 		$linkstart = '<a href="'.$url.'"';
+		if (getDolGlobalInt('MAIN_DISABLE_FORCE_SAVEAS') == 2) {
+			$linkstart .= 'target="_blank" ';
+		}
 		$linkstart .= $linkclose.'>';
 		$linkend = '</a>';
 
 		if ($withpicto) {
-			$result .= ($linkstart.img_object(($notooltip ? '' : $label), 'label', ($notooltip ? '' : 'class="classfortooltip"')).$linkend);
+			if (empty($this->filename)) {
+				$result .= ($linkstart.img_object(($notooltip ? '' : $label), 'label', ($notooltip ? '' : 'class="paddingright"')).$linkend);
+			} else {
+				$result .= ($linkstart.img_mime($this->filename, ($notooltip ? '' : dol_escape_htmltag($label, 1)), ($notooltip ? '' : ' paddingright')).$linkend);
+			}
 			if ($withpicto != 2) {
 				$result .= ' ';
 			}
 		}
-		$result .= $linkstart.$this->ref.$linkend;
+		$result .= $linkstart.$this->filename.$linkend;
 
 		global $action;
 		$hookmanager->initHooks(array($this->element . 'dao'));
-		$parameters = array('id'=>$this->id, 'getnomurl' => &$result);
+		$parameters = array('id' => $this->id, 'getnomurl' => &$result);
 		$reshook = $hookmanager->executeHooks('getNomUrl', $parameters, $this, $action); // Note that $action and $object may have been modified by some hooks
 		if ($reshook > 0) {
 			$result = $hookmanager->resPrint;
 		} else {
 			$result .= $hookmanager->resPrint;
 		}
+
 		return $result;
 	}
 
 	/**
-	 *  Retourne le libelle du status d'un user (actif, inactif)
+	 *  Return the label of the status
 	 *
-	 *  @param	int		$mode          0=libelle long, 1=libelle court, 2=Picto + Libelle court, 3=Picto, 4=Picto + Libelle long, 5=Libelle court + Picto
+	 *  @param  int		$mode          0=long label, 1=short label, 2=Picto + short label, 3=Picto, 4=Picto + long label, 5=Short label + Picto, 6=Long label + Picto
 	 *  @return	string 			       Label of status
 	 */
 	public function getLibStatut($mode = 0)
@@ -923,7 +1105,7 @@ class EcmFiles extends CommonObject
 	 * Initialise object with example values
 	 * Id must be 0 if object instance is a specimen
 	 *
-	 * @return void
+	 * @return int
 	 */
 	public function initAsSpecimen()
 	{
@@ -936,8 +1118,9 @@ class EcmFiles extends CommonObject
 		$this->filename = 'myspecimenfilefile.pdf';
 		$this->filepath = '/aaa/bbb';
 		$this->fullpath_orig = 'c:/file on my disk.pdf';
-		$this->description = 'This is a long description of file';
+		$this->description = 'This is a description of the file';
 		$this->keywords = 'key1,key2';
+		$this->content = 'This is the text content of the file';
 		$this->cover = '1';
 		$this->position = 5;
 		$this->gen_or_uploaded = 'uploaded';
@@ -945,10 +1128,12 @@ class EcmFiles extends CommonObject
 		$this->date_c = (dol_now() - 3600 * 24 * 10);
 		$this->date_m = '';
 		$this->fk_user_c = $user->id;
-		$this->fk_user_m = '';
+		$this->fk_user_m = $user->id;
 		$this->acl = '';
 		$this->src_object_type = 'product';
 		$this->src_object_id = 1;
+
+		return 1;
 	}
 }
 
@@ -956,7 +1141,7 @@ class EcmFiles extends CommonObject
 /**
  * Class of an index line of a document
  */
-class EcmfilesLine
+class EcmFilesLine extends CommonObjectLine
 {
 	/**
 	 * @var string ECM files line label
@@ -968,8 +1153,17 @@ class EcmfilesLine
 	 */
 	public $entity;
 
+	/**
+	 * @var string
+	 */
 	public $filename;
+	/**
+	 * @var string
+	 */
 	public $filepath;
+	/**
+	 * @var string
+	 */
 	public $fullpath_orig;
 
 	/**
@@ -977,12 +1171,39 @@ class EcmfilesLine
 	 */
 	public $description;
 
+	/**
+	 * @var string
+	 */
 	public $keywords;
+
+	/**
+	 * @var string
+	 */
+	public $content;
+
+	/**
+	 * @var string
+	 */
 	public $cover;
+	/**
+	 * @var int
+	 */
 	public $position;
+	/**
+	 * @var 'generated'|'uploaded'|'unknown'|'copy'|''
+	 */
 	public $gen_or_uploaded; // can be 'generated', 'uploaded', 'unknown'
+	/**
+	 * @var string
+	 */
 	public $extraparams;
+	/**
+	 * @var int|''
+	 */
 	public $date_c = '';
+	/**
+	 * @var int|''
+	 */
 	public $date_m = '';
 
 	/**
@@ -995,7 +1216,16 @@ class EcmfilesLine
 	 */
 	public $fk_user_m;
 
+	/**
+	 * @var string
+	 */
 	public $acl;
+	/**
+	 * @var string
+	 */
 	public $src_object_type;
+	/**
+	 * @var int
+	 */
 	public $src_object_id;
 }
