@@ -114,13 +114,13 @@ class Segment implements IteratorAggregate, Countable
 				// Remove the IF tag
 				$this->xml = str_replace('[!-- IF '.$key.' --]', '', $this->xml);
 				// Remove everything between the ELSE tag (if it exists) and the ENDIF tag
-				$reg = '@(\[!--\sELSE\s' . $key . '\s--\](.*))?\[!--\sENDIF\s' . $key . '\s--\]@smU'; // U modifier = all quantifiers are non-greedy
+				$reg = '@(\[!--\sELSE\s' . preg_quote($key, '@') . '\s--\](.*))?\[!--\sENDIF\s' . preg_quote($key, '@') . '\s--\]@smU'; // U modifier = all quantifiers are non-greedy
 				$this->xml = preg_replace($reg, '', $this->xml);
 			}
 			// Else the value is false, then two cases: no ELSE and we're done, or there is at least one place where there is an ELSE clause, then we replace it
 			else {
 				// Find all conditional blocks for this variable: from IF to ELSE and to ENDIF
-				$reg = '@\[!--\sIF\s' . $key . '\s--\](.*)(\[!--\sELSE\s' . $key . '\s--\](.*))?\[!--\sENDIF\s' . $key . '\s--\]@smU'; // U modifier = all quantifiers are non-greedy
+				$reg = '@\[!--\sIF\s' . preg_quote($key, '@') . '\s--\](.*)(\[!--\sELSE\s' . preg_quote($key, '@') . '\s--\](.*))?\[!--\sENDIF\s' . preg_quote($key, '@') . '\s--\]@smU'; // U modifier = all quantifiers are non-greedy
 				preg_match_all($reg, $this->xml, $matches, PREG_SET_ORDER);
 				foreach ($matches as $match) { // For each match, if there is an ELSE clause, we replace the whole block by the value in the ELSE clause
 					if (!empty($match[3])) $this->xml = str_replace($match[0], $match[3], $this->xml);
@@ -180,8 +180,8 @@ class Segment implements IteratorAggregate, Countable
 						 '/__CURRENTMONTH__/u','/__CURRENTMONTHLONG__/u',
 						 '/__NEXTMONTH__/u','/__NEXTMONTHLONG__/u',
 						 '/__CURRENTYEAR__/u','/__NEXTYEAR__/u' );
-		$values=array( $hoy['mday'], $langs->transnoentitiesnoconv($hoy['weekday']),
-					   $hoy['mon'], $langs->transnoentitiesnoconv($hoy['month']),
+		$values=array( $hoy['mday'], $langs->transnoentitiesnoconv($hoy['wday']),
+					   $hoy['mon'], monthArray($langs)[$hoy['mon']],
 					   $nextMonth, monthArray($langs)[$nextMonth],
 					   $hoy['year'], $hoy['year']+1 );
 
